//EHardwareBoneMesh.cpp
#include "EHardwareBoneMesh.h"
#include "../xmlnode/XmlNode.h"
#include "EDefine.h"
#include "XmlMesh.h"
#include "../GameTime.h"


EHardwareBoneMesh::EHardwareBoneMesh( LPDIRECT3DDEVICE9 pDevice ) : EMesh( pDevice )
{
	m_pIndex = NULL;
	m_pMeshVerts = NULL; 
	m_pBoneVerts = NULL;
}
//-----------------------------------------------------------------------------
EHardwareBoneMesh::~EHardwareBoneMesh()
{
	if( m_pMeshVerts )
	{
		delete [] m_pMeshVerts;
		m_pMeshVerts = NULL;
	}

	if( m_pBoneVerts )
	{
		delete [] m_pBoneVerts;
		m_pBoneVerts = NULL;
	}

	if( m_pIndex )
	{
		delete [] m_pIndex;
		m_pIndex = NULL;
	}
}
//-----------------------------------------------------------------------------
const char* EHardwareBoneMesh::GetClassName()
{
	return "EHardwareBoneMesh";
}
//-----------------------------------------------------------------------------
int EHardwareBoneMesh::CreateVertexBuffer( XmlNode* pXmlNode )
{
	XmlMesh* pXmlMesh = NULL;
	if( strcmp( pXmlNode->GetClassName(), "XmlMesh" ) == 0 )
		pXmlMesh = (XmlMesh*)pXmlNode;

	std::vector<VertexInfo>& xmlVx = pXmlMesh->m_vertexArr;

	int n = m_nVertex = (int)xmlVx.size();
	
	if( xmlVx[0].boneIndex.size() == 0 )
	{
		m_pBoneVerts = new HardwareBoneVertex[ m_nVertex ];
		for( int i = 0; i < n; ++i )
		{
			m_pBoneVerts[i].p.x = xmlVx[i].p.x;
			m_pBoneVerts[i].p.y = xmlVx[i].p.y;
			m_pBoneVerts[i].p.z = xmlVx[i].p.z;
		}
	}
	else
	{
		m_pMeshVerts = new HardwareBlendVertex[ m_nVertex ];
		memset( m_pMeshVerts, 0, sizeof(HardwareBlendVertex) * m_nVertex ); 
		for( int i = 0; i < n; ++i )
		{
			m_pMeshVerts[i].p.x = xmlVx[i].p.x;
			m_pMeshVerts[i].p.y = xmlVx[i].p.y;
			m_pMeshVerts[i].p.z = xmlVx[i].p.z;
						
			int count = (int)xmlVx[i].boneIndex.size();
			
			for( int j = 0; j < count; ++j )
			{
				if( j >= 4 )
					break;
				m_pMeshVerts[i].boneIndex[j] =  xmlVx[i].boneIndex[j];	
				if( j < 3 )
					m_pMeshVerts[i].weight[j] = xmlVx[i].boneWeight[j];
			}
		}
	}

	//ε
	m_nTriangle = (DWORD)pXmlMesh->m_indexArr.size();
	m_pIndex = new HardwareFaceIncex [m_nTriangle];
	for( int i = 0 ; i <  (int)m_nTriangle; i++ )
	{
		m_pIndex[i].n[0] = pXmlMesh->m_indexArr[i].n[0];
		m_pIndex[i].n[1] = pXmlMesh->m_indexArr[i].n[1];
		m_pIndex[i].n[2] = pXmlMesh->m_indexArr[i].n[2];
	}

	return 1;
}
//-----------------------------------------------------------------------------
void EHardwareBoneMesh::OnFrameMove()
{
	//Ʈ Ű ִϸ̼ 
	if( this->m_nAniSize > 0 )
	{
		int nFrame = GameTime::GetFrame();	
		m_matWorld = m_aniArray[nFrame];	
	}
	else
		D3DXMatrixIdentity(&m_matWorld);

	if( m_id >= 0 )
		m_matBlend[m_id] = m_matWorld;
}
//-----------------------------------------------------------------------------
int EHardwareBoneMesh::OnRender()
{
	D3DXMATRIXA16 matinit;
	D3DXMatrixIdentity( &matinit );

#if  1	
	// 
	D3DXMATRIXA16 scaleMat;
	D3DXMatrixIdentity( &scaleMat );
	D3DXMatrixScaling( &scaleMat, 0.4f, 0.4f, 0.4f );
#endif

	if( m_pMeshVerts == 0 )
	{
		if( m_id >= 0 )
			m_pDevice->SetTransform(D3DTS_WORLD, &m_matBlend[m_id]);
		return 1;
	}

	m_pDevice->SetSoftwareVertexProcessing( TRUE );
	m_pDevice->SetRenderState( D3DRS_INDEXEDVERTEXBLENDENABLE, TRUE );
	m_pDevice->SetRenderState( D3DRS_VERTEXBLEND, D3DVBF_3WEIGHTS );

	for(int k = 0; k < MAX_BLEND; ++k)
		m_pDevice->SetTransform( D3DTS_WORLDMATRIX(k), &( m_matBlend[k] * scaleMat ) );

	m_pDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME);
	m_pDevice->SetFVF( HardwareBlendVertex::FVF );
	m_pDevice->DrawIndexedPrimitiveUP( D3DPT_TRIANGLELIST, 
		 0, m_nVertex, m_nTriangle, m_pIndex, D3DFMT_INDEX16, m_pMeshVerts, sizeof(HardwareBlendVertex) );

	if( m_pMeshVerts)
	{
		m_pDevice->SetRenderState( D3DRS_INDEXEDVERTEXBLENDENABLE, FALSE);
		m_pDevice->SetRenderState( D3DRS_VERTEXBLEND, D3DVBF_DISABLE  );
		m_pDevice->SetSoftwareVertexProcessing(FALSE);

		for( int k = 0; k < MAX_BLEND; ++k )
			m_pDevice->SetTransform( D3DTS_WORLDMATRIX(k), &matinit);
	}
	m_pDevice->SetTransform( D3DTS_WORLD, &matinit );

	return 1;
}
