//ENode.h

#ifndef _XNODE_H
#define _XNODE_H

#include <d3d9.h>
#include <d3dx9.h>
#include <list>
#include <string>
#include <vector>
#include <map>
#include "EDefine.h"
#include "../tinyxml/tinyxml.h"


class ENode;

typedef std::map<int, ENode*>   IndexENodeMap;

class XmlNode;
class XmlMesh;


#define MAX_BLEND 128

class ENode
{
public:
	ENode( LPDIRECT3DDEVICE9 pDevice );
	virtual	~ENode();

public:
	virtual	const char*	GetClassName();

	ENode*				AttachChild( ENode* pChild );
	void				DetachChild( ENode* pChild );
	ENode*				GetRoot();
	ENode*				GetParent();

	std::string*		GetClassType();
	void				SetClassType( const char* szName );
	std::string*		GetName();
	void				SetName( const char* szName );
	void				SetMatrix( const D3DXMATRIXA16& mat );
	void				CreateAnimationHeader( XmlMesh* pXmlMesh );
	void				CreateAnimation( XmlMesh* pXmlMesh );
	void				SetID( int id )  
	{ 
		m_id = id;  
	};

	const D3DXMATRIXA16&   GetWorldMatrix()  {   return m_matWorld; }

	void				FrameMove();
	//
	void				Render();

public:
	static	IndexENodeMap	m_indexMap;

protected:			
	virtual int			OnRender() { return 1; }
	virtual void		OnFrameMove();

protected:
	typedef std::list <ENode*>	ChildList;

	int						m_id;
	std::string				m_classType;
	std::string				m_name;
	ENode*					m_pParent;
	ChildList				m_child;  
	LPDIRECT3DDEVICE9		m_pDevice;	

	D3DXMATRIXA16			m_matLocal;
	D3DXMATRIXA16			m_matWorld;

	int						m_nAniSize;
	D3DXMATRIXA16*			m_aniArray;		//ִϸ̼ Ʈ		
	AnimationInfo*			m_pAniInfo;

	static D3DXMATRIX		m_matBlend[MAX_BLEND];

};


#endif